@@ -20,6 +20,9 @@
 
 #ifndef SCENE_ITEM_H
 #define SCENE_ITEM_H
+
+#include <CGAL/license/Three.h>
+
 #include <CGAL/Three/Scene_item_config.h>
 #include <CGAL/Three/Scene_interface.h>
 #include <QString>
@@ -29,6 +32,8 @@
 #include <QOpenGLShader>
 #include <QOpenGLVertexArrayObject>
 #include <vector>
+#include <CGAL/Bbox_3.h>
+
 namespace CGAL {
 namespace Three {
   class Viewer_interface;
@@ -43,9 +48,12 @@ class QKeyEvent;
 namespace CGAL {
 namespace Three {
 
+class Scene_group_item;
 class Viewer_interface;
-//! This class represents an object in the OpenGL scene
-class SCENE_ITEM_EXPORT Scene_item : public QObject {
+//! This class represents an object in the OpenGL scene.
+//! It contains all the functions called by the Scene. It
+//! acts like a mix between an interface and a helper.
+class SCENE_ITEM_EXPORT Scene_item : public QObject{
   Q_OBJECT
   Q_PROPERTY(QColor color READ color WRITE setColor)
   Q_PROPERTY(QString name READ name WRITE setName)
@@ -55,213 +63,208 @@ class SCENE_ITEM_EXPORT Scene_item : public QObject {
 public:
  /*!
    * \brief The OpenGL_program_IDs enum
-   * This enum holds the OpenGL programs IDs that are given to getShaderProgram() and attrib_buffers().
-   *@see getShaderProgram
-   * @see attrib_buffers
+   *
+   * This enum holds the OpenGL programs IDs that are given to getShaderProgram() and attribBuffers().
+   * @see getShaderProgram
+   * @see attribBuffers
    */
  enum OpenGL_program_IDs
  {
-  PROGRAM_WITH_LIGHT = 0,      /** Used to render a surface or edge affected by the light. It uses a per fragment lighting model, and renders brighter the selected item.*/
-  PROGRAM_WITHOUT_LIGHT,       /** Used to render a polygon edge or points. It renders in a uniform color and is not affected by light. It renders the selected item in black.*/
-  PROGRAM_NO_SELECTION,        /** Used to render a polyline or a surface that is not affected by light, like a cutting plane. It renders in a uniform color that does not change with selection.*/
-  PROGRAM_WITH_TEXTURE,        /** Used to render a textured polyhedron. Affected by light.*/
-  PROGRAM_PLANE_TWO_FACES,     /** Used to render a two-faced plane. The two faces have a different color. Not affected by light.*/
-  PROGRAM_WITH_TEXTURED_EDGES, /** Used to render the edges of a textured polyhedorn. Not affected by light.*/
-  PROGRAM_INSTANCED,           /** Used to display instanced rendered spheres.Affected by light.*/
-  PROGRAM_INSTANCED_WIRE,      /** Used to display instanced rendered wired spheres. Not affected by light.*/
-  PROGRAM_C3T3,                /** Used to render a c3t3_item. It discards any fragment on a side of a plane, meaning that nothing is displayed on this side of the plane. Affected by light.*/
-  PROGRAM_C3T3_EDGES,          /** Used to render the edges of a c3t3_item. It discards any fragment on a side of a plane, meaning that nothing is displayed on this side of the plane. Not affected by light.*/
-  NB_OF_PROGRAMS               /** Holds the number of different programs in this enum.*/
+  PROGRAM_WITH_LIGHT = 0,      //! Used to render a surface or an edge affected by the light. It uses a per fragment lighting model, and renders the selected item brighter.
+  PROGRAM_WITHOUT_LIGHT,       //! Used to render a polyhedron edge or points. It renders in a uniform color and is not affected by light. \attention It renders the selected item in black.
+  PROGRAM_NO_SELECTION,        //! Used to render a polyline or a surface that is not affected by light, like a cutting plane. It renders in a uniform color that does not change with selection.
+  PROGRAM_WITH_TEXTURE,        //! Used to render a textured polyhedron. Affected by light.
+  PROGRAM_PLANE_TWO_FACES,     //! Used to render a two-faced plane. The two faces have a different color. Not affected by light.
+  PROGRAM_WITH_TEXTURED_EDGES, //! Used to render the edges of a textured polyhedron. Not affected by light.
+  PROGRAM_INSTANCED,           //! Used to display instanced rendered spheres.Affected by light.
+  PROGRAM_INSTANCED_WIRE,      //! Used to display instanced rendered wired spheres. Not affected by light.
+  PROGRAM_C3T3,                //! Used to render a c3t3_item. It discards any fragment on a side of a plane, meaning that nothing is displayed on this side of the plane. Affected by light.
+  PROGRAM_C3T3_EDGES,          //! Used to render the edges of a c3t3_item. It discards any fragment on a side of a plane, meaning that nothing is displayed on this side of the plane. Not affected by light.
+  PROGRAM_CUTPLANE_SPHERES,    //! Used to render the spheres of an item with a cut plane.
+  PROGRAM_SPHERES,             //! Used to render one or several spheres.
+  PROGRAM_C3T3_TETS,           //! Used to render the tetrahedra of the intersection of a c3t3_item.
+  PROGRAM_FLAT,                //! Used to render flat shading without pre computing normals
+  NB_OF_PROGRAMS               //! Holds the number of different programs in this enum.
  };
-  typedef CGAL::Three::Scene_interface::Bbox Bbox;
+  typedef CGAL::Bbox_3 Bbox;
   typedef qglviewer::ManipulatedFrame ManipulatedFrame;
-  //! The default color of a scene_item.
+  //! \brief The default color of a scene_item.
+  //!
+  //! This color is the one that will be displayed if none is specified after its creation.
   static const QColor defaultColor; // defined in Scene_item.cpp
 
-  //!The default Constructor.
-  /*!
-   * Initializes the number of VBOs to 20 and VAOs to 10 and creates them.
-   */
-  Scene_item()
-    : name_("unamed"),
-      color_(defaultColor),
-      visible_(true),
-      are_buffers_filled(false),
-      rendering_mode(FlatPlusEdges),
-      defaultContextMenu(0),
-      buffersSize(20),
-      vaosSize(10),
-      vaos(10)
-  {
-      is_bbox_computed = false;
-      is_monochrome = true;
-      for(int i=0; i<vaosSize; i++)
-      {
-          addVaos(i);
-          vaos[i]->create();
-      }
+  //!\brief The Constructor.
+  //!
+  //! This is where the vectors of VBOs and VAOs are initialized.
+  Scene_item(int buffers_size = 20, int vaos_size = 10);
 
-      buffers.reserve(buffersSize);
-      for(int i=0; i<buffersSize; i++)
-      {
-          QOpenGLBuffer n_buf;
-          buffers.push_back(n_buf);
-          buffers[i].create();
-      }
-      nb_isolated_vertices = 0;
-      has_group = 0;
-  }
-  //!The Constructor.
-  /*!
-   * Initializes the number of VBOs and VAOs and creates them.
-   */
-  Scene_item(int buffers_size, int vaos_size)
-    : name_("unamed"),
-      color_(defaultColor),
-      visible_(true),
-      are_buffers_filled(false),
-      rendering_mode(FlatPlusEdges),
-      defaultContextMenu(0),
-      buffersSize(buffers_size),
-      vaosSize(vaos_size),
-      vaos(vaos_size)
-  {
-      is_bbox_computed = false;
-      is_monochrome = true;
-      for(int i=0; i<vaosSize; i++)
-      {
-          addVaos(i);
-          vaos[i]->create();
-      }
-
-      buffers.reserve(buffersSize);
-      for(int i=0; i<buffersSize; i++)
-      {
-          QOpenGLBuffer n_buf;
-          buffers.push_back(n_buf);
-          buffers[i].create();
-      }
-      nb_isolated_vertices = 0;
-      has_group = 0;
-  }
-  //! Setter for the number of isolated vertices.
+  //! \brief Sets the number of isolated vertices.
+  //!
+  //! This number will be displayed in a warning box at loading.
+  //! @see getNbIsolatedvertices
   void setNbIsolatedvertices(std::size_t nb) { nb_isolated_vertices = nb;}
   //! Getter for the number of isolated vertices.
+  //! @see setNbIsolatedvertices
   std::size_t getNbIsolatedvertices() const {return nb_isolated_vertices;}
-  //!The destructor. It is virtual as the item is virtual.
   virtual ~Scene_item();
-  //! Creates a new item as a copy of the sender. Must be overloaded.
+  //! \brief Duplicates the item.
+  //!
+  //! Creates a new item as a copy of this one.
   virtual Scene_item* clone() const = 0;
 
-  //! Indicates if rendering mode is supported
+  //! \brief Indicates if `m` is supported
+  //!
+  //! If it is, it will be displayed in the context menu of the item.
   virtual bool supportsRenderingMode(RenderingMode m) const = 0;
   //! Deprecated. Does nothing.
   virtual void draw() const {}
-  /*! \brief The drawing function.
-   * Draws the facets of the item in the viewer using OpenGL functions. The data
-   * for the drawing is gathered in compute_elements(), and is sent
-   * to buffers in initialize_buffers().
-   * @see compute_elements()
-   * @see initialize_buffers()
+  /*! \brief The drawing function for faces.
+   *
+   * Draws the faces of the item in the viewer. The data
+   * for the drawing is gathered in computeElements(), and is sent
+   * to buffers in initializeBuffers().
+   * @see computeElements()
+   * @see initializeBuffers()
    */
   virtual void draw(CGAL::Three::Viewer_interface*) const  { draw(); }
   //! Deprecated. Does nothing.
-  virtual void draw_edges() const { draw(); }
-  /*! \brief The drawing function.
-   * Draws the edges and lines of the item in the viewer using OpenGL functions. The data
-   * for the drawing is gathered in compute_elements(), and is sent
-   * to buffers in initialize_buffers().
-   * @see compute_elements()
-   * @see initialize_buffers()
+  virtual void drawEdges() const { draw(); }
+  /*! \brief The drawing function for the edges.
+   *
+   * Draws the edges and lines of the item in the viewer. The data
+   * for the drawing is gathered in computeElements(), and is sent
+   * to buffers in initializeBuffers().
+   * @see computeElements()
+   * @see initializeBuffers()
    */
-  virtual void draw_edges(CGAL::Three::Viewer_interface* viewer) const { draw(viewer); }
+  virtual void drawEdges(CGAL::Three::Viewer_interface* viewer) const { draw(viewer); }
   //! Deprecated. Does nothing.
-  virtual void draw_points() const { draw(); }
-  /*! \brief The drawing function.
-   * Draws the points of the item in the viewer using OpenGL functions. The data
-   * for the drawing is gathered in compute_elements(), and is sent
-   * to buffers in initialize_buffers().
-   * @see compute_elements()
-   * @see initialize_buffers()
+  virtual void drawPoints() const { draw(); }
+  /*! \brief The drawing function for the points.
+   *
+   * Draws the points of the item in the viewer. The data
+   * for the drawing is gathered in computeElements(), and is sent
+   * to buffers in initializeBuffers().
+   * @see computeElements()
+   * @see initializeBuffers()
    */
-  virtual void draw_points(CGAL::Three::Viewer_interface*) const { draw_points(); }
+  virtual void drawPoints(CGAL::Three::Viewer_interface*) const { drawPoints(); }
 
   //! Draws the splats of the item in the viewer using GLSplat functions.
-  virtual void draw_splats() const {}
-  //! Draws the splats of the item in the viewer using GLSplat functions.
-  virtual void draw_splats(CGAL::Three::Viewer_interface*) const {draw_splats();}
+  virtual void drawSplats() const {}
+  //! Draws the splats of the item in the viewer using the GLSplat library.
+  virtual void drawSplats(CGAL::Three::Viewer_interface*) const {drawSplats();}
 
-  //! Specifies which data must be updated when selection has changed.
-  //! Must be overloaded.
-  virtual void selection_changed(bool);
+  //! Called by the scene. If b is true, then this item is currently selected.
+  virtual void selection_changed(bool b);
 
   // Functions for displaying meta-data of the item
+  //!\brief Contains meta-data about the item.
   //! @returns a QString containing meta-data about the item.
-  //! Must be overloaded.
-  //! Data is :Number of vertices, Number of edges, Number of facets,
-  //! Volume, Area, Bounding box limits and Number of isolated points.
   virtual QString toolTip() const = 0;
+  //! \brief Contains graphical meta-data about the item.
   //! @returns a QPixmap containing graphical meta-data about the item.
   virtual QPixmap graphicalToolTip() const { return QPixmap(); }
+  //! \brief Contains the font used for the data of the item.
   //! @returns a QFont containing the font used for the data of the item.
   virtual QFont font() const { return QFont(); }
 
   // Functions that help the Scene to compute its bbox
-  //! If isFinite() returns false, the BBox is not computed.
+  //! \brief Determines if the item is finite or not.
+  //!
+  //! For example, a plane is not finite.
+  //! If false, the BBox is not computed.
   virtual bool isFinite() const { return true; }
   //! Specifies if the item is empty or null.
+  //! If true, the BBox is not computed.
   virtual bool isEmpty() const { return true; }
-  //!@returns the item's bounding box.
+  //! \brief The item's bounding box.
+  //!
+  //! If the Bbox has never been computed, computes it and
+  //! saves the result for further calls.
+  //! @returns the item's bounding box.
   virtual Bbox bbox() const {
       if(!is_bbox_computed)
           compute_bbox();
       is_bbox_computed = true;
       return _bbox;
   }
+  //! \brief the item's bounding box's diagonal length.
+  //!
+  //! If the diagonal's length has never been computed, computes it and
+  //! saves the result for further calls.
+  //! @returns the item's bounding box's diagonal length.
+  virtual double diagonalBbox() const {
+   if(!is_diag_bbox_computed)
+       compute_diag_bbox();
+   is_diag_bbox_computed = true;
+   return _diag_bbox;
+  }
+
   // Function about manipulation
-  //! Decides if the item can have a ManipulatedFrame.
+  //! Returns true if the item has a ManipulatedFrame.
+  //! @see manipulatedFrame()
   virtual bool manipulatable() const { return false; }
+  //!\brief The manipulatedFrame of the item.
+  //!
+  //! A manipulated frame is an independant system that can be
+  //! translated or rotated using the Ctrl key and the mouse.
   //!@returns the manipulatedFrame of the item.
   virtual ManipulatedFrame* manipulatedFrame() { return 0; }
 
   // Getters for the four basic properties
+  //!Getter for the item's color.
   //! @returns the current color of the item.
   virtual QColor color() const { return color_; }
+  //!Getter for the item's name.
   //! @returns the current name of the item.
   virtual QString name() const { return name_; }
+  //! If the item is not visible, it is not drawn and its Bbox
+  //! is ignored in the computation of the scene's.
   //! @returns the current visibility of the item.
   virtual bool visible() const { return visible_; }
+  //!Getter for the item's rendering mode.
   //! @returns the current rendering mode of the item.
   //!@see RenderingMode
   virtual RenderingMode renderingMode() const { return rendering_mode; }
+  //!The renderingMode's name.
   //! @returns the current rendering mode of the item as a human readable string.
   virtual QString renderingModeName() const;
 
-  //! Context menu
+  //! \brief Context menu
+  //!
+  //! Contains the list of the supported rendering modes,
+  //! the Operations menu, actions to save or clone the item if it is supported
+  //! and any contextual action for the item.
   virtual QMenu* contextMenu();
 
   //!Handles key press events.
   virtual bool keyPressEvent(QKeyEvent*){return false;}
 
+  //!The group containing the item.
+  //! \returns the parent group if the item is in a group
+  //! \returns 0 if the item is not in a group.
+  Scene_group_item* parentGroup() const;
+
   //!Contains the header for the table in the statistics dialog
   /*!
    * A header data is composed of 2 columns : the Categories and the titles.
    * A category is the name given to an association of titles.
    * A title is the name of a line.
-   *
+   *\verbatim
    * For example,
    * Category :    | Titles| Values
    * 2 lines       |       |
    *  ____________________________
    * |             |Name   |Cube |
    * |             |_______|_____|
-   * |General Info |\#Edges|12   |
+   * |General Info | #Edges|12   |
    * |_____________|_______|_____|
    *
    *  would be stored as follows :
    * categories = std::pair<QString,int>(QString("General Info"),2)
    * titles.append("Name");
-   * titles.append("#Edges");
+   * titles.append("#Edges");\endverbatim
    */
   struct Header_data{
    //!Contains the name of the category of statistics and the number of lines it will contain
@@ -271,20 +274,22 @@ public:
   };
   //!Returns a Header_data struct containing the header information.
   virtual Header_data header()const;
+  //!Returns true if the item has statistics.
+  virtual bool has_stats()const{return false;}
   //!Returns a QString containing the requested value for the the table in the statistics dialog
-  /*!
+  /*! \verbatim
    * Example :
    *  ____________________________
    * |             |Name   |Cube |
    * |             |_______|_____|
-   * |General Info |\#Edges|12   |
+   * |General Info | #Edges|12   |
    * |_____________|_______|_____|
-   * compute stats(0) should return "Cube" and compute_stats(1) should return QString::number(12);
+   * compute stats(0) should return "Cube" and computeStats(1) should return QString::number(12);
    * The numbers must be coherent with the order of declaration of the titles in the header.
-   *
+   * \endverbatim
    *
    */
-  virtual QString compute_stats(int i);
+  virtual QString computeStats(int i);
 
   //!Contains the number of group and subgroups containing this item.
   int has_group;
@@ -292,37 +297,44 @@ public:
 public Q_SLOTS:
 
   //! Notifies the program that the internal data or the properties of
-  //! an item has changed, and that it must be computed again.It is
+  //! an item has changed, and that it must be computed again. It is
   //! important to call this function whenever the internal data is changed,
   //! or the displayed item will not be updated.
-  //!Must be overloaded.
   virtual void invalidateOpenGLBuffers();
-  //!Setter for the color of the item. Calls invalidateOpenGLBuffers() so the new color is applied.
-  virtual void setColor(QColor c) { color_ = c; if(!is_monochrome)invalidateOpenGLBuffers(); }
+  //!Setter for the color of the item.
+  virtual void setColor(QColor c) { color_ = c;}
   //!Setter for the RGB color of the item. Calls setColor(QColor).
   //!@see setColor(QColor c)
   void setRbgColor(int r, int g, int b) { setColor(QColor(r, g, b)); }
-  //!Setter for the name of the item.
+  //!Sets the name of the item.
   virtual void setName(QString n) { name_ = n; }
-  //!Setter for the visibility of the item.
-  virtual void setVisible(bool b) { visible_ = b; }
-  //!Setter for the rendering mode of the item.
+    //!Sets the visibility of the item.
+  virtual void setVisible(bool b);
+  //!Set the parent group. If `group==0`, then the item has no parent.
+  //!This function is called by `Scene::changeGroup` and should not be
+  //!called manually.
+  virtual void moveToGroup(Scene_group_item* group);
+  //!Sets the rendering mode of the item.
   //!@see RenderingMode
   virtual void setRenderingMode(RenderingMode m) { 
     if (supportsRenderingMode(m))
       rendering_mode = m; 
     Q_EMIT redraw();
   }
-  //!Set the RenderingMode to Points.
+  //!Sets the RenderingMode to Points.
   void setPointsMode() {
     setRenderingMode(Points);
   }
-  //!Set the RenderingMode to Wireframe.
+  //!Sets the RenderingMode to Points.
+  void setShadedPointsMode() {
+    setRenderingMode(ShadedPoints);
+  }
+  //!Sets the RenderingMode to Wireframe.
   void setWireframeMode() {
     setRenderingMode(Wireframe);
   }
 
-  //!Set the RenderingMode to Flat.
+  //!Sets the RenderingMode to Flat.
   void setFlatMode() {
     setRenderingMode(Flat);
   }
@@ -330,27 +342,23 @@ public Q_SLOTS:
   void setFlatPlusEdgesMode() {
     setRenderingMode(FlatPlusEdges);
   }
-  //!Set the RenderingMode to Gouraud.
+  //!Sets the RenderingMode to Gouraud.
   void setGouraudMode() {
     setRenderingMode(Gouraud);
   }
-  //!Set the RenderingMode to PointsPlusNormals.
+  //!Sets the RenderingMode to PointsPlusNormals.
   void setPointsPlusNormalsMode(){
     setRenderingMode(PointsPlusNormals);
   }
-  //!Set the RenderingMode to Splatting.
+  //!Sets the RenderingMode to Splatting.
   void setSplattingMode(){
     setRenderingMode(Splatting);
   }
-
-  //! If b is true, the item will use buffers to render the color.
-  //! If b is false, it will use a uniform value. For example, when
-  //! using the mesh segmentation plugin, the item must be multicolor.
-  void setItemIsMulticolor(bool b){
-    is_monochrome = !b;
-  }
   
   //!Emits an aboutToBeDestroyed() signal.
+  //!Override this function to delete what needs to be deleted on destruction.
+  //!This might be needed as items are not always deleted right away by Qt and this behaviour may cause a simily
+  //!memory leak, for example when multiple items are created at the same time.
   virtual void itemAboutToBeDestroyed(Scene_item*);
 
   //!Selects a point through raycasting.
@@ -361,16 +369,27 @@ public Q_SLOTS:
                       double dir_y,
                       double dir_z);
 
+
+
 Q_SIGNALS:
+  //! Is emitted to notify a change in the item's data.
   void itemChanged();
+  //! Is emitted when the item is shown to notify a change in the item's visibility.
+  //! Typically used to update the scene's bbox;
+  void itemVisibilityChanged();
+  //! Is emitted to notify that the item is about to be deleted.
   void aboutToBeDestroyed();
+  //! Is emitted to require a new display.
   void redraw();
 
 protected:
   //!Holds the BBox of the item
   mutable Bbox _bbox;
+  mutable double _diag_bbox;
   mutable bool is_bbox_computed;
+  mutable bool is_diag_bbox_computed;
   virtual void compute_bbox()const{}
+  virtual void compute_diag_bbox()const;
   // The four basic properties
   //!The name of the item.
   QString name_;
@@ -378,19 +397,18 @@ protected:
   QColor color_;
   //!The visibility of the item.
   bool visible_;
+  //!The parent group, or 0 if the item is not in a group.
+  Scene_group_item* parent_group;
   //!Specifies if the item is currently selected.
   bool is_selected;
-  //! Specifies if the item is monochrome and uses uniform attribute for its color
-  //! or is multicolor and uses buffers.
-  bool is_monochrome;
   //! Holds the number of vertices that are not linked to the polyhedron from the OFF
   //! file.
   std::size_t nb_isolated_vertices;
-  /*! Decides if the draw function must call initialize_buffers() or not. It is set
-   * to true in the end of initialize_buffers() and to false in invalidateOpenGLBuffers(). The need of
+  /*! Decides if the draw function must call initializeBuffers() or not. It is set
+   * to true in the end of initializeBuffers() and to false in invalidateOpenGLBuffers(). The need of
    * this boolean comes from the need of a context from the OpenGLFunctions used in
-   * initialize_buffers().
-   * @see initialize_buffers()
+   * initializeBuffers().
+   * @see initializeBuffers()
    * @see invalidateOpenGLBuffers()
    */
   mutable bool are_buffers_filled;
@@ -405,8 +423,8 @@ protected:
    * @see invalidateOpenGLBuffers()*/
   RenderingMode prev_shading;
   /*! \todo replace it by RenderingMode().
-   * \brief
-   *  Contains the current RenderingMode.
+   * \brief Contains the current RenderingMode.
+   *
    * This is used to determine if invalidateOpenGLBuffers should be called or not
    * in certain cases.
    * @see invalidateOpenGLBuffers()*/
@@ -427,25 +445,19 @@ protected:
       vaos[i] = n_vao;
   }
 
-  //! Used pass data to the shader.
-  int vertexLoc;
-  //! Used pass data to the shader.
-  int normalLoc;
-  //! Used pass data to the shader.
-  int colorLoc;
-  /*! Fills the VBOs with data. Must be called after each call to #compute_elements().
+  /*! Fills the VBOs with data. Must be called after each call to #computeElements().
    * @see compute_elements()
    */
-  void initialize_buffers(){}
+  void initializeBuffers(){}
 
   /*! Collects all the data for the shaders. Must be called in #invalidateOpenGLBuffers().
    * @see invalidateOpenGLBuffers().
    */
-  void compute_elements(){}
+  void computeElements(){}
   /*! Passes all the uniform data to the shaders.
    * According to program_name, this data may change.
    */
-  void attrib_buffers(CGAL::Three::Viewer_interface*, int program_name) const;
+  void attribBuffers(CGAL::Three::Viewer_interface*, int program_name) const;
 
   /*! Compatibility function. Calls `viewer->getShaderProgram()`. */
   virtual QOpenGLShaderProgram* getShaderProgram(int name , CGAL::Three::Viewer_interface *viewer = 0) const;
