@@ -26,9 +26,11 @@
 #include <CGAL/config.h>
 #include <CGAL/export/helpers.h>
 
-#if defined(CGAL_BUILD_SHARED_LIBS)
+#if ( defined(CGAL_BUILD_SHARED_LIBS) && ( ! defined(CGAL_HEADER_ONLY) ) ) \
+  || defined(CGAL_USE_Qt5_RESOURCES)
 
-#  if defined(CGAL_Qt5_EXPORTS) // defined by CMake or in cpp files of the dll
+#  if defined(CGAL_Qt5_EXPORTS) || defined(CGAL_USE_Qt5_RESOURCES)
+// defined by CMake or in cpp files of the dll
 
 #    define CGAL_QT_EXPORT CGAL_DLL_EXPORT
 #    define CGAL_QT_EXPIMP_TEMPLATE
